@@ -174,6 +174,7 @@ clist_initialize_device_procs(gx_device *dev)
     set_dev_proc(dev, copy_alpha_hl_color, clist_copy_alpha_hl_color);
     set_dev_proc(dev, process_page, clist_process_page);
     set_dev_proc(dev, fill_stroke_path, clist_fill_stroke_path);
+    set_dev_proc(dev, lock_pattern, clist_lock_pattern);
 }
 
 /*------------------- Choose the implementation -----------------------
@@ -708,6 +709,7 @@ clist_open(gx_device *dev)
 errxit:
     /* prevent leak */
     gs_free_object(cdev->memory->non_gc_memory, cdev->cache_chunk, "free tile cache for clist");
+    dev->is_open = save_is_open;
     cdev->cache_chunk = NULL;
     return code;
 }
@@ -1543,5 +1545,7 @@ open_c:
             pdev->buf = NULL;
         }
     }
+    if (code < 0)
+        pdev->is_open = save_is_open;
     return code;
 }
